#include "Status.h"
#include "Vram.h"
#include "Stage.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}


static void PrintRemain(ptr<byte> pVram)
{
    byte c;
    c = Char_Fighter;
    repeat (2) {
        repeat (2) {
            VPut(pVram, c);
            pVram += 1;
            ++c;
        }
        pVram += VramWidth - 2;
    }
}

void PrintStatus() 
{
    PrintS(Vram + VramWidth * 0 + 32, "SCORE");
    PrintS(Vram + VramWidth * 4 + 32, "HI-SCORE");
    PrintS(Vram + VramWidth * 8 + 32, "STAGE");
    {
        ptr<byte> pVram;
        pVram = Vram + VramWidth * 21 + 33;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                PrintRemain(pVram);
                pVram += 3;
                VPut(pVram, Char_Space);
                PrintC(pVram + VramWidth, i + '0');
                ++pVram;
            }
            else {
                do {
                    PrintRemain(pVram);
                    pVram += 3;
                    --i;
                } while (i > 0);
            }
        }
        repeat (2) {
            repeat (2) {
                VPut(pVram, Char_Space);
                pVram += 1;
            }
            pVram += VramWidth - 2;
        }                
    }
    PrintScore();
    PrintStage();
}

void PrintScore()
{
    ptr<byte> pVram;
    
    pVram = PrintNumber5(Vram + VramWidth * 2 + 34, Score);
    PrintC(pVram, '0');

    pVram = PrintNumber5(Vram + VramWidth * 5 + 34, HiScore);
    PrintC(pVram, '0');
}

void PrintStage()
{
    PrintByteNumber3(Vram + VramWidth * 8 + 37, CurrentStage + 1);
}

static void EraseWindow() 
{
    constexpr byte width = 12;
    constexpr byte height = 3;    
    ptr<byte> pVram;

    pVram = Vram + VramRowSize * 11 + 10;
    repeat (height) {
        repeat (width) {
            pVram = PrintC(pVram, ' ');
        }
        pVram += VramRowSize - width;
    }
}

void PrintGameOver()
{
    EraseWindow();
    PrintS(Vram + VramWidth * 11 + 12, "GAME OVER");
}

void Title()
{
    static const byte[] TitleBytes = {
        //	I
        0x04, 0x0d, 0x07, 0x01, 
        0x00, 0x0c, 0x03, 0x00, 
        0x00, 0x0c, 0x03, 0x00, 
        0x04, 0x05, 0x05, 0x01, 
        //	M
        0x0c, 0x07, 0x07, 0x0b, 
        0x0c, 0x03, 0x03, 0x0f, 
        0x0c, 0x03, 0x03, 0x0f, 
        0x04, 0x01, 0x01, 0x05, 
        //	P
        0x0c, 0x07, 0x05, 0x0b, 
        0x0c, 0x03, 0x00, 0x0f, 
        0x0c, 0x07, 0x05, 0x01, 
        0x04, 0x01, 0x00, 0x00, 
        //	E
        0x0c, 0x07, 0x05, 0x01, 
        0x0c, 0x0b, 0x0a, 0x02, 
        0x0c, 0x03, 0x00, 0x00, 
        0x04, 0x05, 0x05, 0x01, 
        //	T
        0x04, 0x0d, 0x07, 0x01, 
        0x00, 0x0c, 0x03, 0x00, 
        0x00, 0x0c, 0x03, 0x00, 
        0x00, 0x04, 0x01, 0x00, 
        //	U
        0x0c, 0x03, 0x00, 0x0f, 
        0x0c, 0x03, 0x00, 0x0f, 
        0x0c, 0x03, 0x00, 0x0f, 
        0x00, 0x05, 0x05, 0x01, 
        //	S
        0x08, 0x07, 0x05, 0x0b, 
        0x04, 0x0b, 0x0a, 0x02, 
        0x08, 0x02, 0x00, 0x0f, 
        0x00, 0x05, 0x05, 0x01, 
    };
    ClearScreen(); 
    HideAllSprites();
    PrintStatus();
{
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = Vram + VramRowSize * 7 + 2;
        p = TitleBytes;
        repeat (7) {
            repeat (4) {
                repeat (4) {
                    VPut(pVram, *p);
                    ++pVram;
                    ++p;
                }
                pVram += VramRowSize - 4;
            }
            pVram += 4 - VramRowSize * 4;
        }
    }
    PrintS(Vram + VramRowSize * 19 + 9, "PUSH SPACE KEY");
    PrintS(Vram + VramRowSize * 20 + 9, "OR SHIFT KEY");
}
